package tech.nevets.codebadgerestapi;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.batik.svggen.SVGGraphics2DIOException;
import org.slf4j.LoggerFactory;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;

public class BadgeFactory {

    public static byte[] genSVGBadge(String leftStr, String rightStr, Color secondaryColor) {
        DOMImplementation domImpl = GenericDOMImplementation.getDOMImplementation();
        Document doc = domImpl.createDocument("http://www.w3.org/2000/svg", "svg", null);
        SVGGraphics2D graphics2D = new SVGGraphics2D(doc);
        Font font = new Font("Calibri", Font.PLAIN, 16);

        graphics2D.setFont(font);
        FontMetrics fm = graphics2D.getFontMetrics();
        int lWidth = fm.stringWidth(leftStr);
        int rWidth = fm.stringWidth(rightStr);
        int totalWidth = lWidth = lWidth + rWidth + 40;
        int totalHeight = 20;
        graphics2D.setPaint(Color.GRAY);
        graphics2D.fillRect(0, 0, totalWidth, totalHeight);
        graphics2D.setPaint(secondaryColor);
        graphics2D.fillRect(totalWidth - rWidth - 20, 0, rWidth + 20, totalHeight);
        graphics2D.setPaint(Color.GRAY);
        graphics2D.drawString(leftStr, 10, (totalHeight / 2) + 6);
        graphics2D.drawString(rightStr, totalWidth - rWidth - 10, (totalHeight / 2) + 6);
        graphics2D.setPaint(Color.WHITE);
        graphics2D.drawString(leftStr, 10, (totalHeight / 2) + 5);
        graphics2D.drawString(rightStr, totalWidth - rWidth - 10, (totalHeight / 2) + 5);

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        OutputStreamWriter osw = new OutputStreamWriter(baos);
        try {
            graphics2D.stream(osw, true);
        } catch (SVGGraphics2DIOException e) {
            e.printStackTrace();
            return new byte[]{};
        }
        return baos.toByteArray();
    }

    public static byte[] generateBadge(String leftStr, String rightStr, BadgeColor secondaryColor) {
        BufferedImage img = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
        Graphics2D graphics2D = img.createGraphics();
        Font font = new Font("Calibri", Font.PLAIN, 23);
        graphics2D.setFont(font);
        FontMetrics fontMetrics = graphics2D.getFontMetrics();
        int lWidth = fontMetrics.stringWidth(leftStr);
        int rWidth = fontMetrics.stringWidth(rightStr);
        int height = fontMetrics.getHeight();
        graphics2D.dispose();

        int totalWidth = lWidth + rWidth + 20;
        int totalHeight = 25;
        int textY = 19;

        BufferedImage baseBadgeImg;
        BufferedImage secondaryImg;
        try {
            baseBadgeImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-base-gray.png"));
            switch (secondaryColor) {
                case RED -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-color-red.png"));
                case ORANGE -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-color-orange.png"));
                case YELLOW -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-color-yellow.png"));
                case GREEN -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-color-green.png"));
                case BLUE -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-color-blue.png"));
                case PURPLE -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-color-purple.png"));
                case LIGHTGRAY -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-color-light-gray.png"));
                default -> secondaryImg = ImageIO.read(BadgeFactory.class.getResourceAsStream("/badge-base-gray.png"));
            }
        } catch (IOException e) {
            e.printStackTrace();
            baseBadgeImg = null;
            secondaryImg = null;
        }

        img = new BufferedImage(totalWidth, totalHeight, BufferedImage.TYPE_INT_ARGB);
        graphics2D = img.createGraphics();
        graphics2D.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
        graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        graphics2D.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        graphics2D.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE);
        graphics2D.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
        graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        graphics2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        graphics2D.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);
        graphics2D.setFont(font);
        graphics2D.drawImage(resizeImage(baseBadgeImg, totalWidth), 0, 0, totalWidth, totalHeight, null);
        graphics2D.drawImage(secondaryImg, totalWidth - rWidth - 10, 0, rWidth + 10, totalHeight, null);
        graphics2D.setColor(Color.GRAY);
        graphics2D.drawString(leftStr, 5, textY - 1);
        graphics2D.drawString(rightStr, totalWidth - rWidth - 5, textY - 1);
        graphics2D.setColor(Color.WHITE);
        graphics2D.drawString(leftStr, 5, textY);
        graphics2D.drawString(rightStr, totalWidth - rWidth - 5, textY);
        graphics2D.dispose();

        ByteArrayOutputStream baos;
        try {
            baos = new ByteArrayOutputStream();
            ImageIO.write(img, "png", baos);
        } catch (IOException e) {
            e.printStackTrace();
            LoggerFactory.getLogger(BadgeFactory.class).info("not good thingy happened...");
            return new byte[]{};
        }

        return baos.toByteArray();
    }

    private static BufferedImage resizeImage(BufferedImage originalImage, int targetWidth) {
        BufferedImage resizedImage = new BufferedImage(targetWidth, 25, BufferedImage.TYPE_INT_ARGB);
        Graphics2D graphics2D = resizedImage.createGraphics();
        graphics2D.drawImage(originalImage, 0, 0, targetWidth, 25, null);
        graphics2D.dispose();
        return resizedImage;
    }

    public static String getVersionFromGitRepo(String gitUrl, String org, String repo, String branch) {
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .GET()
                .header("accept", "text/plain")
                .uri(URI.create("https://" + gitUrl + "/" + org + "/" + repo + "/raw/branch/" + branch + "/build.gradle"))
                .build();
        String responseString = "";
        try {
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            responseString = response.body();
        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        }

        String[] split = responseString.split("\n");
        String version = null;
        for (String str : split) {
            if (str.startsWith("version")) {
                version = str.substring(9, str.length() - 1);
            }
        }

        return version;
    }

    public static String getLatestBuildStatusFromJenkins(String jenkinsUrl, String projectName) {
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .GET()
                .header("accept", "application/json")
                .uri(URI.create("https://" + jenkinsUrl + "/job/" + projectName + "/lastBuild/api/json"))
                .build();
        String responseString = "";
        try {
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            responseString = response.body();
        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        }

        JsonObject json = new Gson().fromJson(responseString, JsonObject.class);
        return json.get("result").getAsString();
    }

    public static String getLatestSuccessfulVersionFromJenkins(String jenkinsUrl, String projectName) {
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .GET()
                .header("accept", "application/json")
                .uri(URI.create("https://" + jenkinsUrl + "/job/" + projectName + "/lastSuccessfulBuild/api/json"))
                .build();
        String responseString = "";
        try {
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            responseString = response.body();
        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        }

        JsonObject json = new Gson().fromJson(responseString, JsonObject.class);
        String gitUrl = json.get("actions").getAsJsonArray().get(2).getAsJsonObject().get("remoteUrls").getAsJsonArray().get(0).getAsString();
        gitUrl = gitUrl.substring(0, gitUrl.length() - 4);
        String commit = json.get("changeSet").getAsJsonObject().get("items").getAsJsonArray().get(0).getAsJsonObject().get("commitId").getAsString();

        HttpClient gitClient = HttpClient.newHttpClient();
        HttpRequest gitRequest = HttpRequest.newBuilder()
                .GET()
                .header("accept", "text/plain")
                .uri(URI.create(gitUrl + "/raw/commit/" + commit + "/build.gradle"))
                .build();
        String gitResponseString = "";
        try {
            HttpResponse<String> response = gitClient.send(gitRequest, HttpResponse.BodyHandlers.ofString());
            gitResponseString = response.body();
        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        }

        String[] split = gitResponseString.split("\n");
        String version = null;
        for (String str : split) {
            if (str.startsWith("version")) {
                version = str.substring(9, str.length() - 1);
            }
        }

        return version;
    }
}
