package tech.nevets.codebadgerestapi;

import java.awt.*;

import static spark.Spark.*;

public class Server {
    public static void main(String[] args) {
        path("/badge", () -> {
            before("/*");
            get("/test/:left/:right/:color", (req, res) -> {
                res.type("image/svg+xml");
                Color color = Color.GRAY;
                switch (req.params(":color").toLowerCase()) {
                    case "red" -> color = Color.RED;
                    case "orange" -> color = Color.ORANGE;
                    case "yellow" -> color = Color.YELLOW;
                    case "green" -> color = Color.GREEN;
                    case "cyan" -> color = Color.CYAN;
                    case "blue" -> color = Color.BLUE;
                    case "purple", "magenta" -> color = Color.MAGENTA;
                    case "pink" -> color = Color.PINK;
                    case "white" -> color = Color.WHITE;
                    case "lightgray" -> color = Color.LIGHT_GRAY;
                    case "darkgray" -> color = Color.DARK_GRAY;
                    case "black" -> color = Color.BLACK;
                    default -> color = Color.GRAY;
                }
                return BadgeFactory.genSVGBadge(req.params(":left"), req.params(":right"), color);
            });
            get("/version/*/:gitUrl/:org/:repo/:branch", (req, res) -> {
                switch (req.splat()[0]) {
                    case "gitea" -> {
                        return BadgeFactory.getVersionFromGitRepo(req.params(":gitUrl"), req.params(":org"), req.params(":repo"), req.params(":branch"));
                    }
                    case "github" -> {
                        res.status(405);
                        return "Not yet Implemented!";
                    }
                    case "gitlab" -> {
                        res.status(405);
                        return "Not yet Implemented!";
                    }
                    case "test" -> {
                        res.type("image/png");
                        String version = BadgeFactory.getVersionFromGitRepo(req.params(":gitUrl"), req.params(":org"), req.params(":repo"), req.params(":branch"));
                        return BadgeFactory.generateBadge("Version", version, BadgeColor.GREEN);
                    }
                }
                res.status(404);
                return "Not Found!";
            });

            get("/buildStatus/*/:ciUrl/:projectName", (req, res) -> {
                switch (req.splat()[0]) {
                    case "jenkins" -> {
                        res.type("image/png");
                        String buildStatus = BadgeFactory.getLatestBuildStatusFromJenkins(req.params(":ciUrl"), req.params(":projectName"));
                        BadgeColor badgeColor = BadgeColor.GRAY;
                        switch (buildStatus) {
                            case "SUCCESS" -> {
                                buildStatus = "passing";
                                badgeColor = BadgeColor.GREEN;
                            }
                            case "FAILURE" -> {
                                buildStatus = "failing";
                                badgeColor = BadgeColor.RED;
                            }
                        }
                        return BadgeFactory.generateBadge("Build", buildStatus, badgeColor);
                    }
                    case "circleci" -> {
                        res.status(405);
                        return "Not yet Implemented!";
                    }
                    case "teamcity" -> {
                        res.status(405);
                        return "Not yet Implemented!";
                    }
                    case "test" -> {
                        res.type("image/svg+xml");

                    }
                }
                res.status(404);
                return "Not Found!";
            });

            get("/latestSuccessfulVersion/*/:ciUrl/:projectName", (req, res) -> {
                switch (req.splat()[0]) {
                    case "jenkins" -> {
                        return BadgeFactory.getLatestSuccessfulVersionFromJenkins(req.params(":ciUrl"), req.params(":projectName"));
                    }
                    case "circleci" -> {
                        res.status(405);
                        return "Not yet Implemented!";
                    }
                    case "teamcity" -> {
                        res.status(405);
                        return "Not yet Implemented!";
                    }
                }
                res.status(404);
                return "Not Found!";
            });
        });
    }
}
